module DocMachine::Container

  class DockerEngine < AbstractContainerEngine
    # Pulls the specified Docker image
    def pull_image(image_tag : String) : Nil
      Log.info { "Pulling Docker image: #{image_tag}" }
      status = Process.run("docker", ["pull", image_tag], output: STDOUT, error: STDERR)
      if status.success?
        Log.info { "Successfully pulled Docker image: #{image_tag}" }
        true
      else
        Log.error { "Failed to pull Docker image: #{image_tag}" }
        false
      end
    end

    # Loads the Docker image from a local cache
    def load_image(cache_path : String) : Nil
      Log.info { "Loading Docker image from cache: #{cache_path}" }
      status = Process.run("docker", ["image", "load", "-i", cache_path], output: STDOUT, error: STDERR)
      if status.success?
        Log.info { "Successfully loaded Docker image from cache." }
        true
      else
        Log.error { "Failed to load Docker image from cache." }
        false
      end
    end

    # Saves the Docker image to a local cache
    def save_image(image_tag : String, cache_path : String) : Nil
      Log.info { "Saving Docker image #{image_tag} to cache at #{cache_path}" }
      status = Process.run("docker", ["image", "save", image_tag, "-o", cache_path], output: STDOUT, error: STDERR)
      if status.success?
        Log.info { "Successfully saved Docker image to cache." }
        true
      else
        Log.error { "Failed to save Docker image to cache." }
        false
      end
    end

    # Runs a Docker container with the given options
    def run_container(
      image_tag : String,
      action : String,
      docker_name : String,
      docker_opts : Array(String),
      enable_tty : Bool
    ) : Process
      Log.info { "Running Docker container: #{docker_name}" }
      
      # Construct the full Docker run command
      cmd = ["docker", "run"] + docker_opts + [image_tag, action]
      
      # Log the command for debugging
      Log.debug { "Docker run command: #{cmd.join(" ")}" }
      
      # Start the Docker container process
      process = Process.new("docker", ["run"] + docker_opts + [image_tag, action], input: STDIN, output: STDOUT, error: STDERR)
      
      Log.info { "Docker container #{docker_name} started." }
      
      process
    end

    # Kills a running Docker container by name
    def kill_container(container_name : String) : Nil
      Log.info { "Killing Docker container: #{container_name}" }
      status = Process.run("docker", ["kill", container_name], output: STDOUT, error: STDERR)
      if status.success?
        Log.info { "Successfully killed Docker container: #{container_name}" }
        true
      else
        Log.error { "Failed to kill Docker container: #{container_name}" }
        false
      end
    end

    # Removes a Docker container by name
    def remove_container(container_name : String) : Nil
      Log.info { "Removing Docker container: #{container_name}" }
      status = Process.run("docker", ["rm", container_name], output: STDOUT, error: STDERR)
      if status.success?
        Log.info { "Successfully removed Docker container: #{container_name}" }
        true
      else
        Log.error { "Failed to remove Docker container: #{container_name}" }
        false
      end
    end

    # Finds the container ID based on its name
    def find_container_id(name : String) : String
      Log.info { "Finding Docker container ID for name: #{name}" }
      output = IO::Memory.new
      status = Process.run("docker", ["ps", "-f", "name=#{name}", "-q"], output: output, error: STDERR)
      if status.success?
        container_id = output.to_s.strip
        Log.info { "Found Docker container ID: #{container_id}" }
        container_id
      else
        Log.error { "Failed to find Docker container ID for name: #{name}" }
        ""
      end
    end
  end

end
